unit CiaDataCall;

interface

uses
  Classes, ExtCtrls;

type
   TCiaDataCallState = (dcsReady, dcsDialing, dcsConnected, dcsDisconnecting, dcsHangup);
const
   aDataCallState: array[TCiaDataCallState] of string =
                    ('Ready', 'Dialing', 'Connected', 'Disconnecting', 'Hangup');

type
  TOnData = procedure(Sender: TObject; const Data: string) of object;
  TOnTimeOut = procedure(Sender: TObject; OldState: TCiaDataCallState; var AutoHandle: boolean) of object;
  TOnChangeState = procedure(Sender: TObject; NewState: TCiaDataCallState) of object;
  TCiaDataCall = class(TComponent)
  private
    FDataCallState: TCiaDataCallState;
    FOnSend: TOnData;
    FOnDataAvailable: TOnData;
    FOnConnect: TNotifyEvent;
    FOnClosed: TNotifyEvent;
    FOnTimeOut: TOnTimeOut;
    FOnChangeState: TOnChangeState;
    FTimeOutCarrier: integer;
    FTimeOutKeepAlive: integer;
    FTimeOutHangup: integer;
    FTimer: TTimer;
    FKeepAlive: boolean;
    procedure ToPhone(const Data: String);
    procedure TimerTimer(Sender: TObject);
    procedure StartTimer(TimeOut: integer);
    procedure StopTimer;
    procedure Hangup;
    procedure Close;
    procedure ChangeState(NewState: TCiaDataCallState);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Connect(const Phone: string);
    procedure ReceiveStr(const Data: string);
    procedure SendStr(const Data: string);
    procedure Disconnect;
    property  State: TCiaDataCallState read FDataCallState;
  published
    property KeepAlive: boolean read FKeepAlive write FKeepAlive;
    property TimeOutCarrier: integer read FTimeOutCarrier write FTimeOutCarrier;
    property TimeOutKeepAlive: integer read FTimeOutKeepAlive write FTimeOutKeepAlive;
    property TimeOutHangup: integer read FTimeOutHangup write FTimeOutHangup;
    property OnChangeState: TOnChangeState read FOnChangeState write FOnChangeState;
    property OnClosed: TNotifyEvent read FOnClosed write FOnClosed;
    property OnConnect: TNotifyEvent read FOnConnect write FOnConnect;
    property OnDataAvailable: TOnData read FOnDataAvailable write FOnDataAvailable;
    property OnTimeOut: TOnTimeOut read FOnTimeOut write FOnTimeOut;
    property OnSend: TOnData read FOnSend write FOnSend;
  end;

procedure Register;

implementation

//------------------------------------------------------------------------------
function CmpStr(const SearchBegin, ToSearch: string; Exact: boolean = True): boolean;
var n: integer;
begin
   n := 1;
   if Exact then
      while (SearchBegin[n] = ToSearch[n]) and (ToSearch[n] <> #0) do
         inc(n)
   else
      while ( UpCase(SearchBegin[n]) = UpCase(ToSearch[n]) ) and ( ToSearch[n] <> #0 ) do
         inc(n);
   Result := ToSearch[n] = #0;
end;

//------------------------------------------------------------------------------
//---- TCiaDataCall ------------------------------------------------------------
//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('Cia', [TCiaDataCall]);
end;

//------------------------------------------------------------------------------
constructor TCiaDataCall.Create(AOwner: TComponent);
begin
   inherited;
   FTimeOutCarrier   := 60000;
   FTimeOutKeepAlive := 30000;
   FTimeOutHangup    := 10000;
   FKeepAlive := True;
   FTimer     := TTimer.Create(nil);
   with FTimer do
   begin
      Enabled  := False;
      OnTimer  := TimerTimer;
   end;
end;

//------------------------------------------------------------------------------
destructor TCiaDataCall.Destroy;
begin
   FTimer.Destroy;
   inherited;
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.StopTimer;
begin
   FTimer.Enabled := False;
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.StartTimer(TimeOut: integer);
begin
   StopTimer;
   FTimer.Interval := TimeOut;
   FTimer.Enabled  := True;
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.ToPhone(const Data: string);
begin
   if Assigned(FOnSend) then
      FOnSend(Self, Data);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.SendStr(const Data: string);
begin
   ToPhone(Data);
   if FKeepAlive then
      StartTimer(FTimeOutKeepAlive);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.ChangeState(NewState: TCiaDataCallState);
begin
   FDataCallState := NewState;
   if Assigned(FOnChangeState) then
      FOnChangeState(Self, NewState);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.Connect(const Phone: string);
begin
   ChangeState(dcsDialing);
   ToPhone('ATDT' + Phone + #13#10);
   StartTimer(FTimeOutCarrier);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.Disconnect;
begin
   ChangeState(dcsDisconnecting);
   ToPhone('+++');
   StartTimer(FTimeOutHangup);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.Hangup;
begin
   ChangeState(dcsHangup);
   ToPhone('ATH0'#13#10);
   StartTimer(FTimeOutHangup);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.Close;
begin
   StopTimer;
   if Assigned(FOnClosed) then
      FOnClosed(Self);
   ChangeState(dcsReady);
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.TimerTimer(Sender: TObject);
var
   AutoHandle: boolean;
begin
   StopTimer;

   AutoHandle := True;
   if Assigned(FOnTimeOut) then
      FOnTimeOut(Self, FDataCallState, AutoHandle);

   if AutoHandle then
      case FDataCallState of
         dcsDialing:
            Disconnect;

         dcsConnected:
            if FKeepAlive then
               Disconnect;

         dcsDisconnecting:
            Hangup;

         dcsHangup:
            Close;
      end;
end;

//------------------------------------------------------------------------------
procedure TCiaDataCall.ReceiveStr(const Data: string);
begin
   case FDataCallState of
      dcsDialing:
         if CmpStr(Data, 'CONNECT') then
         begin
            ChangeState(dcsConnected);
            if Assigned(FOnConnect) then
               FOnConnect(Self);
            if FKeepAlive then
               StartTimer(FTimeOutKeepAlive)
            else
               StopTimer;
         end
         else if CmpStr(Data, 'NO CARRIER') or
                 CmpStr(Data, 'BUSY')       then
            Close;

      dcsConnected:
      begin
         if Assigned(FOnDataAvailable) then
            FOnDataAvailable(Self, Data);
         if FKeepAlive then
            StartTimer(FTimeOutKeepAlive)
         else
            StopTimer;
      end;

      dcsDisconnecting:
         if CmpStr(Data, 'OK') then
            Hangup;

      dcsHangup:
         if CmpStr(Data, 'OK') then
            Close;
   end;
end;

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
end.
